'use strict';

var redux = require('redux');

function _defineProperty(obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
}

function _objectSpread(target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i] != null ? arguments[i] : {};
    var ownKeys = Object.keys(source);

    if (typeof Object.getOwnPropertySymbols === 'function') {
      ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) {
        return Object.getOwnPropertyDescriptor(source, sym).enumerable;
      }));
    }

    ownKeys.forEach(function (key) {
      _defineProperty(target, key, source[key]);
    });
  }

  return target;
}

function createEffectMiddleware(context) {
  return function (_ref) {
    var dispatch = _ref.dispatch,
        getState = _ref.getState;
    return function (next) {
      return function (action) {
        var effects = context.effects;

        if (typeof effects[action.type] === 'function') {
          return effects[action.type](action, dispatch, getState);
        }

        return next(action);
      };
    };
  };
}

function _createStore (reducer, initialState, externalMiddlewares) {
  if (reducer === void 0) {
    reducer = function reducer(state) {
      return state;
    };
  }

  if (initialState === void 0) {
    initialState = {};
  }

  if (externalMiddlewares === void 0) {
    externalMiddlewares = [];
  }

  var middlewares = externalMiddlewares.concat([createEffectMiddleware(this)]);
  var enhancers = [redux.applyMiddleware.apply(void 0, middlewares)]; // If Redux DevTools Extension is installed use it, otherwise use Redux compose

  /* istanbul ignore next */

  var composeEnhancers = process.env.NODE_ENV !== 'production' && typeof window === 'object' && window.__REDUX_DEVTOOLS_EXTENSION_COMPOSE__ // eslint-disable-line
  ? window.__REDUX_DEVTOOLS_EXTENSION_COMPOSE__({
    // eslint-disable-line
    shouldHotReload: false
  }) : redux.compose;
  var store = redux.createStore(reducer, initialState, composeEnhancers.apply(void 0, enhancers));
  return store;
}

var reducerHelper = {
  getReducer: function getReducer(reducers, initialState) {
    if (initialState === void 0) {
      initialState = null;
    }

    return function (state, action) {
      if (state === void 0) {
        state = initialState;
      }

      if (typeof reducers[action.type] === 'function') {
        return reducers[action.type](state, action);
      }

      return state;
    };
  },
  createReducer: function createReducer(injectedReducers) {
    return redux.combineReducers(_objectSpread({}, injectedReducers));
  }
};

var constants = {
  SEP: '/',
  SET_STATE: 'setState'
};

var SEP = constants.SEP;

function actionCreator(namespace, actionName, dispatch) {
  return function (payload) {
    return dispatch({
      type: "" + namespace + SEP + actionName,
      payload: payload
    });
  };
}

var actionHelper = {
  add: function add(model, dispatch) {
    var actions = {};
    var _model$reducers = model.reducers,
        reducers = _model$reducers === void 0 ? {} : _model$reducers,
        _model$effects = model.effects,
        effects = _model$effects === void 0 ? {} : _model$effects,
        namespace = model.namespace;

    var validate = function validate(actionName) {
      if (actions[actionName]) {
        throw new Error("Action name \"" + namespace + SEP + actionName + "\" has been duplicate defined! Please select another action name!");
      }
    };

    Object.keys(reducers).forEach(function (actionName) {
      validate(actionName);
      actions[actionName] = actionCreator(namespace, actionName, dispatch);
    });
    Object.keys(effects).forEach(function (actionName) {
      validate(actionName);
      actions[actionName] = actionCreator(namespace, actionName, dispatch);
    });
    return actions;
  }
};

var SEP$1 = constants.SEP;
function prefixNamespace(namespace, items) {
  if (items === void 0) {
    items = {};
  }

  return Object.keys(items).reduce(function (acc, cur) {
    acc["" + namespace + SEP$1 + cur] = items[cur];
    return acc;
  }, {});
}
var isArray = Array.isArray.bind(Array);
var isPlainObject = require('is-plain-object'); // eslint-disable-line

var SET_STATE = constants.SET_STATE;
var getReducer = reducerHelper.getReducer,
    createReducer = reducerHelper.createReducer;

function filterReducers(reducers) {
  if (!reducers) {
    return reducers;
  }

  return Object.keys(reducers).reduce(function (acc, action) {
    if (typeof reducers[action] === 'function') {
      acc[action] = reducers[action];
    }

    return acc;
  }, {});
}

function validateModel(model, models) {
  if (model === void 0) {
    model = {};
  }

  var _model = model,
      namespace = _model.namespace,
      initialState = _model.initialState,
      reducers = _model.reducers,
      effects = _model.effects;

  if (!namespace || typeof namespace !== 'string') {
    throw new Error("Expected the model namespace to be a string. but got " + typeof namespace);
  }

  if (models.some(function (item) {
    return item.namespace === namespace;
  })) {
    throw new Error("Model \"" + namespace + "\" has been created, please select another name!");
  }

  if (reducers !== undefined && !isPlainObject(reducers)) {
    throw new Error("Expected the model reducers to be a plain object, but got " + typeof reducers);
  }

  if (effects !== undefined && !isPlainObject(effects)) {
    throw new Error("Expected the model effects to be a plain object, but got " + typeof effects);
  }

  return {
    namespace: namespace,
    initialState: initialState,
    reducers: filterReducers(reducers),
    effects: filterReducers(effects)
  };
}

var XRedux =
/*#__PURE__*/
function () {
  function XRedux() {
    this.store = null;
    this.actions = {};
    this.models = [];
    this.MEMORY_MODELS = [];
    this.reducers = {};
    this.effects = {};
    this.createStore = this.createStore.bind(this);
    this.model = this.model.bind(this);
  }

  var _proto = XRedux.prototype;

  _proto.createStore = function createStore(reducers, initialState, externalMiddlewares) {
    if (externalMiddlewares && !isArray(externalMiddlewares)) {
      throw new Error("Expected the middlewares to be a array, but got " + typeof externalMiddlewares);
    } // create store


    var store = _createStore.call(this, reducers ? redux.combineReducers(reducers) : undefined, initialState, externalMiddlewares); // init reducer


    if (reducers) {
      this.reducers = _objectSpread({}, reducers);
    } // save store


    this.store = store; // init memory models

    if (this.MEMORY_MODELS.length > 0) {
      this.addModels(this.MEMORY_MODELS);
      this.MEMORY_MODELS = [];
    }

    return store;
  };

  _proto.addModels = function addModels(models) {
    if (models && !isArray(models)) {
      throw new Error("Expected the models to be a array, but got " + typeof models);
    }

    for (var i = 0; i < models.length; i += 1) {
      var m = models[i];
      var model = validateModel(m, this.models);
      var namespace = model.namespace;

      if (model.reducers && Object.keys(model.reducers).indexOf(SET_STATE) === -1) {
        // inject setState reducer to reducers
        // it can help to set state quickly in the effects
        model.reducers[SET_STATE] = function (state, action) {
          return _objectSpread({}, state, action.payload);
        };
      } // prefix namespace to reducers and effects


      var reducers = prefixNamespace(namespace, model.reducers);
      var effects = prefixNamespace(namespace, model.effects); // inject reducer

      var finalReducer = getReducer(reducers, model.initialState);
      this.reducers[namespace] = finalReducer; // add effects

      this.effects = _objectSpread({}, this.effects, effects); // add actions

      var actions = actionHelper.add(model, this.store.dispatch);
      this.actions[namespace] = actions; // save model

      this.models.push(model);
    } // update reducer in the store


    this.store.replaceReducer(createReducer(this.reducers));
  };

  _proto.model = function model(m) {
    if (!this.store) {
      this.MEMORY_MODELS.push(m);
      return;
    }

    this.addModels([m]);
  };

  return XRedux;
}();

function createInstance() {
  var instance = new XRedux();
  return instance;
}

var xredux = createInstance();

xredux.create = function () {
  return createInstance();
};

module.exports = xredux;
